@@ -12,12 +12,21 @@ decision<-function(tau,null,alt=NA,penalty="SIC",n=0,diffparam=1,value=0){
 	if((penalty=="SIC") || (penalty=="BIC")){
 		value=diffparam*log(n)
 	}
+	else if((penalty=="SIC1") || (penalty=="BIC1")){
+		value=(diffparam+1)*log(n)
+	}
 	else if(penalty=="AIC"){
 		value=2*diffparam
 	}
+	else if(penalty=="AIC1"){
+		value=2*(diffparam+1)
+	}
 	else if(penalty=="Hannan-Quinn"){
 		value=2*diffparam*log(log(n))
 	}
+	else if(penalty=="Hannan-Quinn1"){
+		value=2*(diffparam+1)*log(log(n))
+	}
 	else if(penalty=="None"){
 		value=0
 	}
