@@ -302,12 +302,21 @@ multiple.meanvar.gamma=function(data,shape=1,mul.method="PELT",penalty="SIC",val
 	if((penalty=="SIC") || (penalty=="BIC")){
 		value=diffparam*log(n)
 	}
+	else if((penalty=="SIC1") || (penalty=="BIC1")){
+		value=(diffparam+1)*log(n)
+	}
 	else if(penalty=="AIC"){
 		value=2*diffparam
 	}
+	else if(penalty=="AIC1"){
+		value=2*(diffparam+1)
+	}
 	else if(penalty=="Hannan-Quinn"){
 		value=2*diffparam*log(log(n))
 	}
+	else if(penalty=="Hannan-Quinn1"){
+		value=2*(diffparam+1)*log(log(n))
+	}
 	else if(penalty=="None"){
 		value=0
 	}
